@@ -1,3 +1,4 @@
+/* $Id: ClpPackedMatrix.hpp 1458 2009-11-05 12:34:07Z forrest $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
 #ifndef ClpPackedMatrix_H
@@ -22,21 +23,21 @@ public:
    //@{
    /// Return a complete CoinPackedMatrix
   virtual CoinPackedMatrix * getPackedMatrix() const { return matrix_;}
-    /** Whether the packed matrix is column major ordered or not. */
-    virtual bool isColOrdered() const { return matrix_->isColOrdered(); }
+  /** Whether the packed matrix is column major ordered or not. */
+  virtual bool isColOrdered() const { return matrix_->isColOrdered(); }
    /** Number of entries in the packed matrix. */
   virtual  CoinBigIndex getNumElements() const 
   { return matrix_->getNumElements(); }
-   /** Number of columns. */
-   virtual int getNumCols() const { return matrix_->getNumCols(); }
-   /** Number of rows. */
+  /** Number of columns. */
+  virtual int getNumCols() const { return matrix_->getNumCols(); }
+  /** Number of rows. */
   virtual int getNumRows() const { return matrix_->getNumRows(); }
-
-   /** A vector containing the elements in the packed matrix. Note that there
-	might be gaps in this list, entries that do not belong to any
-	major-dimension vector. To get the actual elements one should look at
-	this vector together with vectorStarts and vectorLengths. */
-   virtual const double * getElements() const 
+  
+  /** A vector containing the elements in the packed matrix. Note that there
+      might be gaps in this list, entries that do not belong to any
+      major-dimension vector. To get the actual elements one should look at
+      this vector together with vectorStarts and vectorLengths. */
+  virtual const double * getElements() const 
   { return matrix_->getElements();}
   /// Mutable elements
   inline double * getMutableElements() const
@@ -91,9 +92,7 @@ public:
   /** Returns a new matrix in reverse order without gaps */
   virtual ClpMatrixBase * reverseOrderedCopy() const;
   /// Returns number of elements in column part of basis 
-  virtual CoinBigIndex countBasis(ClpSimplex * model,
-				 const int * whichColumn, 
-				 int numberRowBasic,
+  virtual CoinBigIndex countBasis(const int * whichColumn, 
 				  int & numberColumnBasic);
   /// Fills in column part of basis
   virtual void fillBasis(ClpSimplex * model,
@@ -101,10 +100,10 @@ public:
 				 int & numberColumnBasic,
 				 int * row, int * start,
 				 int * rowCount, int * columnCount,
-				 double * element);
+				 CoinFactorizationDouble * element);
   /** Creates scales for column copy (rowCopy in model may be modified)
       returns non-zero if no scaling done */
-  virtual int scale(ClpModel * model, const ClpSimplex * baseModel=NULL) const ;
+  virtual int scale(ClpModel * model,const ClpSimplex * baseModel=NULL) const ;
   /** Scales rowCopy if column copy scaled
       Only called if scales already exist */
   virtual void scaleRowCopy(ClpModel * model) const ;
@@ -198,6 +197,16 @@ public:
 				const double * rowScale, 
 				const double * columnScale,
 				double * spare=NULL) const;
+    /** Return <code>y - pi * A</code> in <code>y</code>.
+        @pre <code>pi</code> must be of size <code>numRows()</code>
+        @pre <code>y</code> must be of size <code>numColumns()</code> 
+	This just does subset (but puts in correct place in y) */
+  void transposeTimesSubset( int number,
+			     const int * which,
+			     const double * pi, double * y,
+			     const double * rowScale, 
+			     const double * columnScale,
+			     double * spare=NULL) const;
     /** Return <code>x * scalar * A + y</code> in <code>z</code>. 
 	Can use y as temporary array (will be empty at end)
 	Note - If x packed mode - then z packed mode
@@ -237,7 +246,7 @@ public:
   /// Updates two arrays for steepest 
   virtual void transposeTimes2(const ClpSimplex * model,
                                const CoinIndexedVector * pi1, CoinIndexedVector * dj1,
-                               const CoinIndexedVector * pi2, CoinIndexedVector * dj2,
+                               const CoinIndexedVector * pi2, 
                                CoinIndexedVector * spare,
                                double referenceIn, double devex,
                                // Array for exact devex to say what is in reference framework
@@ -246,13 +255,20 @@ public:
   /// Updates second array for steepest and does devex weights 
   virtual void subsetTimes2(const ClpSimplex * model,
                                 CoinIndexedVector * dj1,
-                               const CoinIndexedVector * pi2, CoinIndexedVector * dj2,
+                               const CoinIndexedVector * pi2, CoinIndexedVector * dj2, 
                                double referenceIn, double devex,
                                // Array for exact devex to say what is in reference framework
                                unsigned int * reference,
                                double * weights, double scaleFactor);
   /// Sets up an effective RHS
   void useEffectiveRhs(ClpSimplex * model);
+#if COIN_LONG_WORK 
+  // For long double versions 
+  virtual void times(CoinWorkDouble scalar,
+		     const CoinWorkDouble * x, CoinWorkDouble * y) const ;
+  virtual void transposeTimes(CoinWorkDouble scalar,
+			      const CoinWorkDouble * x, CoinWorkDouble * y) const ;
+#endif
 //@}
 
   /**@name Other */
@@ -266,7 +282,7 @@ public:
   { matrix_=NULL;}
   /// Say we want special column copy
   inline void makeSpecialColumnCopy()
-  { flags_ |= 8;}
+  { flags_ |= 16;}
   /// Say we don't want special column copy
   void releaseSpecialColumnCopy();
   /// Are there zeros?
@@ -274,10 +290,13 @@ public:
   { return ((flags_&1)!=0);}
   /// Do we want special column copy
   inline bool wantsSpecialColumnCopy() const
-  { return ((flags_&8)!=0);}
+  { return ((flags_&16)!=0);}
   /// Flags
   inline int flags() const
   { return flags_;}
+  /// Sets flags_ correctly
+  inline void checkGaps()
+  { flags_ = (matrix_->hasGaps()) ? (flags_|2) : (flags_&(~2));} 
    //@}
 
 
@@ -336,6 +355,41 @@ private:
 				 int * COIN_RESTRICT index, 
 				 double * COIN_RESTRICT array,
 				 const double tolerance) const;
+  /// Meat of transposeTimes by column when not scaled and skipping
+  int gutsOfTransposeTimesUnscaled(const double * COIN_RESTRICT pi,
+				    int * COIN_RESTRICT index, 
+				    double * COIN_RESTRICT array,
+				   const unsigned char * status,
+				    const double tolerance) const;
+  /** Meat of transposeTimes by column when not scaled and skipping
+      and doing part of dualColumn */
+  int gutsOfTransposeTimesUnscaled(const double * COIN_RESTRICT pi,
+				    int * COIN_RESTRICT index, 
+				    double * COIN_RESTRICT array,
+				   const unsigned char * status,
+				    int * COIN_RESTRICT spareIndex, 
+				    double * COIN_RESTRICT spareArray,
+				    const double * COIN_RESTRICT reducedCost,
+				   double & upperTheta,
+				   double & bestPossible,
+				   double acceptablePivot,
+				   double dualTolerance,
+				   int & numberRemaining,
+				    const double zeroTolerance) const;
+  /// Meat of transposeTimes by column when scaled and skipping
+  int gutsOfTransposeTimesScaled(const double * COIN_RESTRICT pi,
+				 const double * COIN_RESTRICT columnScale,
+				 int * COIN_RESTRICT index, 
+				 double * COIN_RESTRICT array,
+				   const unsigned char * status,
+				 const double tolerance) const;
+  /// Meat of transposeTimes by row n > K if packed - returns number nonzero
+  int gutsOfTransposeTimesByRowGEK(const CoinIndexedVector * COIN_RESTRICT piVector, 
+				   int * COIN_RESTRICT index, 
+ 				   double * COIN_RESTRICT output,
+				   int numberColumns,
+				   const double tolerance, 
+				   const double scalar) const;
   /// Meat of transposeTimes by row n > 2 if packed - returns number nonzero
   int gutsOfTransposeTimesByRowGE3(const CoinIndexedVector * COIN_RESTRICT piVector, 
 				   int * COIN_RESTRICT index, 
@@ -369,6 +423,7 @@ protected:
       2 - has gaps
       4 - has special row copy
       8 - has special column copy
+      16 - wants special column copy
   */
   int flags_;
   /// Special row copy
